@@ -1,15 +1,15 @@
 #ifndef _BUFFER_H_
 #define _BUFFER_H_
 
-#include <stdlib.h>
-#include <sys/types.h>
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
 #include "settings.h"
 
+#include <stdlib.h>
+#include <sys/types.h>
+
 typedef struct {
 	char *ptr;
 
@@ -103,7 +103,7 @@ int buffer_to_lower(buffer *b);
 int buffer_to_upper(buffer *b);
 
 /** deprecated */
-int ltostr(char *buf, long val);
+int LI_ltostr(char *buf, long val);
 char hex2int(unsigned char c);
 char int2hex(char i);
 
