@@ -1,9 +1,14 @@
-#include <string.h>
-#include <stdint.h>
-
 #include "buffer.h"
 #include "etag.h"
 
+#if defined HAVE_STDINT_H
+# include <stdint.h>
+#elif defined HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
+
+#include <string.h>
+
 int etag_is_equal(buffer *etag, const char *matches) {
 	if (etag && !buffer_is_empty(etag) && 0 == strcmp(etag->ptr, matches)) return 1;
 	return 0;
@@ -35,7 +40,7 @@ int etag_mutate(buffer *mut, buffer *etag) {
 	size_t i;
 	uint32_t h;
 
-	for (h=0, i=0; i < etag->used; ++i) h = (h<<5)^(h>>27)^(etag->ptr[i]);
+	for (h=0, i=0; i < etag->used-1; ++i) h = (h<<5)^(h>>27)^(etag->ptr[i]);
 
 	buffer_reset(mut);
 	buffer_copy_string_len(mut, CONST_STR_LEN("\""));
