@@ -1,6 +1,7 @@
-#include <sys/types.h>
+#include "fdevent.h"
+#include "buffer.h"
 
-#include "settings.h"
+#include <sys/types.h>
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -9,8 +10,6 @@
 #include <stdio.h>
 #include <fcntl.h>
 
-#include "fdevent.h"
-#include "buffer.h"
 
 fdevents *fdevent_init(size_t maxfds, fdevent_handler_t type) {
 	fdevents *ev;
@@ -92,7 +91,7 @@ int fdevent_reset(fdevents *ev) {
 	return 0;
 }
 
-fdnode *fdnode_init() {
+static fdnode *fdnode_init() {
 	fdnode *fdn;
 
 	fdn = calloc(1, sizeof(*fdn));
@@ -100,7 +99,7 @@ fdnode *fdnode_init() {
 	return fdn;
 }
 
-void fdnode_free(fdnode *fdn) {
+static void fdnode_free(fdnode *fdn) {
 	free(fdn);
 }
 
