@@ -1,6 +1,9 @@
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "server.h"
+#include "log.h"
+#include "http_auth.h"
+#include "http_auth_digest.h"
+#include "inet_ntop_cache.h"
+#include "stream.h"
 
 #ifdef HAVE_CRYPT_H
 # include <crypt.h>
@@ -25,13 +28,6 @@
 #include <unistd.h>
 #include <ctype.h>
 
-#include "server.h"
-#include "log.h"
-#include "http_auth.h"
-#include "http_auth_digest.h"
-#include "inet_ntop_cache.h"
-#include "stream.h"
-
 #ifdef USE_OPENSSL
 # include <openssl/md5.h>
 #else
@@ -728,10 +724,15 @@ static int http_auth_basic_password_compare(server *srv, mod_auth_plugin_data *p
 			char c = username->ptr[i];
 
 			if (!isalpha(c) &&
-			    !isdigit(c)) {
+			    !isdigit(c) &&
+			    (c != ' ') &&
+			    (c != '@') &&
+			    (c != '-') &&
+			    (c != '_') &&
+			    (c != '.') ) {
 
 				log_error_write(srv, __FILE__, __LINE__, "sbd",
-					"ldap: invalid character (a-zA-Z0-9 allowed) in username:", username, i);
+					"ldap: invalid character (- _.@a-zA-Z0-9 allowed) in username:", username, i);
 
 				return -1;
 			}
@@ -865,7 +866,11 @@ int http_auth_basic_check(server *srv, connection *con, mod_auth_plugin_data *p,
 		buffer_free(username);
 		buffer_free(password);
 
-		log_error_write(srv, __FILE__, __LINE__, "s", "get_password failed");
+		if (AUTH_BACKEND_UNSET == p->conf.auth_backend) {
+			log_error_write(srv, __FILE__, __LINE__, "s", "auth.backend is not set");
+		} else {
+			log_error_write(srv, __FILE__, __LINE__, "s", "get_password failed");
+		}
 
 		return 0;
 	}
@@ -909,15 +914,15 @@ int http_auth_digest_check(server *srv, connection *con, mod_auth_plugin_data *p
 	char a1[256];
 	char a2[256];
 
-	char *username;
-	char *realm;
-	char *nonce;
-	char *uri;
-	char *algorithm;
-	char *qop;
-	char *cnonce;
-	char *nc;
-	char *respons;
+	char *username = NULL;
+	char *realm = NULL;
+	char *nonce = NULL;
+	char *uri = NULL;
+	char *algorithm = NULL;
+	char *qop = NULL;
+	char *cnonce = NULL;
+	char *nc = NULL;
+	char *respons = NULL;
 
 	char *e, *c;
 	const char *m = NULL;
@@ -958,15 +963,9 @@ int http_auth_digest_check(server *srv, connection *con, mod_auth_plugin_data *p
 	dkv[6].ptr = &cnonce;
 	dkv[7].ptr = &nc;
 	dkv[8].ptr = &respons;
-	dkv[9].ptr = NULL;
 
 	UNUSED(req);
 
-	for (i = 0; dkv[i].key; i++) {
-		*(dkv[i].ptr) = NULL;
-	}
-
-
 	if (p->conf.auth_backend != AUTH_BACKEND_HTDIGEST &&
 	    p->conf.auth_backend != AUTH_BACKEND_PLAIN) {
 		log_error_write(srv, __FILE__, __LINE__, "s",
@@ -1184,6 +1183,7 @@ int http_auth_digest_generate_nonce(server *srv, mod_auth_plugin_data *p, buffer
 	/* we assume sizeof(time_t) == 4 here, but if not it ain't a problem at all */
 	LI_ltostr(hh, srv->cur_ts);
 	MD5_Update(&Md5Ctx, (unsigned char *)hh, strlen(hh));
+	MD5_Update(&Md5Ctx, (unsigned char *)srv->entropy, sizeof(srv->entropy));
 	LI_ltostr(hh, rand());
 	MD5_Update(&Md5Ctx, (unsigned char *)hh, strlen(hh));
 
