@@ -1,14 +1,23 @@
+#include "server.h"
+#include "stat_cache.h"
+#include "keyvalue.h"
+#include "log.h"
+#include "connections.h"
+#include "joblist.h"
+#include "http_chunk.h"
+
+#include "plugin.h"
+
 #include <sys/types.h>
+
 #ifdef __WIN32
-#include <winsock2.h>
+# include <winsock2.h>
 #else
-#include <sys/socket.h>
-#include <sys/wait.h>
-#include <sys/mman.h>
-
-#include <netinet/in.h>
-
-#include <arpa/inet.h>
+# include <sys/socket.h>
+# include <sys/wait.h>
+# include <sys/mman.h>
+# include <netinet/in.h>
+# include <arpa/inet.h>
 #endif
 
 #include <unistd.h>
@@ -23,19 +32,12 @@
 #include <stdio.h>
 #include <fcntl.h>
 
-#include "server.h"
-#include "keyvalue.h"
-#include "log.h"
-#include "connections.h"
-#include "joblist.h"
-#include "http_chunk.h"
-
-#include "plugin.h"
-
 #ifdef HAVE_SYS_FILIO_H
 # include <sys/filio.h>
 #endif
 
+#include "version.h"
+
 enum {EOL_UNSET, EOL_N, EOL_RN};
 
 typedef struct {
@@ -53,6 +55,7 @@ typedef struct {
 
 typedef struct {
 	array *cgi;
+	unsigned short execute_x_only;
 } plugin_config;
 
 typedef struct {
@@ -148,6 +151,7 @@ SETDEFAULTS_FUNC(mod_fastcgi_set_defaults) {
 
 	config_values_t cv[] = {
 		{ "cgi.assign",                  NULL, T_CONFIG_ARRAY, T_CONFIG_SCOPE_CONNECTION },       /* 0 */
+		{ "cgi.execute-x-only",          NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION },     /* 1 */
 		{ NULL,                          NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET}
 	};
 
@@ -162,8 +166,10 @@ SETDEFAULTS_FUNC(mod_fastcgi_set_defaults) {
 		assert(s);
 
 		s->cgi    = array_init();
+		s->execute_x_only = 0;
 
 		cv[0].destination = s->cgi;
+		cv[1].destination = &(s->execute_x_only);
 
 		p->config_storage[i] = s;
 
@@ -577,12 +583,9 @@ static handler_t cgi_connection_close(server *srv, handler_ctx *hctx) {
 #if 0
 				log_error_write(srv, __FILE__, __LINE__, "sd", "(debug) cgi exited fine, pid:", pid);
 #endif
-				pid = 0;
-
 				return HANDLER_GO_ON;
 			} else {
 				log_error_write(srv, __FILE__, __LINE__, "sd", "cgi died, pid:", pid);
-				pid = 0;
 				return HANDLER_GO_ON;
 			}
 		}
@@ -696,11 +699,11 @@ static int cgi_env_add(char_array *env, const char *key, size_t key_len, const c
 
 	if (!key || !val) return -1;
 
-	dst = malloc(key_len + val_len + 3);
+	dst = malloc(key_len + val_len + 2);
 	memcpy(dst, key, key_len);
 	dst[key_len] = '=';
-	/* add the \0 from the value */
-	memcpy(dst + key_len + 1, val, val_len + 1);
+	memcpy(dst + key_len + 1, val, val_len);
+	dst[key_len + 1 + val_len] = '\0';
 
 	if (env->size == 0) {
 		env->size = 16;
@@ -744,6 +747,8 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 	}
 
 	if (pipe(from_cgi_fds)) {
+		close(to_cgi_fds[0]);
+		close(to_cgi_fds[1]);
 		log_error_write(srv, __FILE__, __LINE__, "ss", "pipe failed:", strerror(errno));
 		return -1;
 	}
@@ -776,25 +781,23 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 		/* not needed */
 		close(to_cgi_fds[1]);
 
-		/* HACK:
-		 * this is not nice, but it works
-		 *
-		 * we feed the stderr of the CGI to our errorlog, if possible
-		 */
-		if (srv->errorlog_mode == ERRORLOG_FILE) {
-			close(STDERR_FILENO);
-			dup2(srv->errorlog_fd, STDERR_FILENO);
-		}
-
 		/* create environment */
 		env.ptr = NULL;
 		env.size = 0;
 		env.used = 0;
 
-		cgi_env_add(&env, CONST_STR_LEN("SERVER_SOFTWARE"), CONST_STR_LEN(PACKAGE_NAME"/"PACKAGE_VERSION));
+		if (buffer_is_empty(con->conf.server_tag)) {
+			cgi_env_add(&env, CONST_STR_LEN("SERVER_SOFTWARE"), CONST_STR_LEN(PACKAGE_DESC));
+		} else {
+			cgi_env_add(&env, CONST_STR_LEN("SERVER_SOFTWARE"), CONST_BUF_LEN(con->conf.server_tag));
+		}
 
 		if (!buffer_is_empty(con->server_name)) {
-			cgi_env_add(&env, CONST_STR_LEN("SERVER_NAME"), CONST_BUF_LEN(con->server_name));
+			size_t len = con->server_name->used - 1;
+			char *colon = strchr(con->server_name->ptr, ':');
+			if (colon) len = colon - con->server_name->ptr;
+
+			cgi_env_add(&env, CONST_STR_LEN("SERVER_NAME"), con->server_name->ptr, len);
 		} else {
 #ifdef HAVE_IPV6
 			s = inet_ntop(srv_sock->addr.plain.sa_family,
@@ -972,9 +975,15 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 				buffer_prepare_append(p->tmp_buf, ds->key->used + 2);
 
 				for (j = 0; j < ds->key->used - 1; j++) {
-					p->tmp_buf->ptr[p->tmp_buf->used++] =
-						light_isalnum((unsigned char)ds->key->ptr[j]) ?
-						toupper((unsigned char)ds->key->ptr[j]) : '_';
+					char cr = '_';
+					if (light_isalpha(ds->key->ptr[j])) {
+						/* upper-case */
+						cr = ds->key->ptr[j] & ~32;
+					} else if (light_isdigit(ds->key->ptr[j])) {
+						/* copy */
+						cr = ds->key->ptr[j];
+					}
+					p->tmp_buf->ptr[p->tmp_buf->used++] = cr;
 				}
 				p->tmp_buf->ptr[p->tmp_buf->used++] = '\0';
 
@@ -998,7 +1007,7 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 			args[i++] = cgi_handler->ptr;
 		}
 		args[i++] = con->physical.path->ptr;
-		args[i++] = NULL;
+		args[i  ] = NULL;
 
 		/* search for the last / */
 		if (NULL != (c = strrchr(con->physical.path->ptr, '/'))) {
@@ -1011,8 +1020,6 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 			*c = '/';
 		}
 
-		openDevNull(STDERR_FILENO);
-
 		/* we don't need the client socket */
 		for (i = 3; i < 256; i++) {
 			if (i != srv->errorlog_fd) close(i);
@@ -1030,6 +1037,10 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 	case -1:
 		/* error */
 		log_error_write(srv, __FILE__, __LINE__, "ss", "fork failed:", strerror(errno));
+		close(from_cgi_fds[0]);
+		close(from_cgi_fds[1]);
+		close(to_cgi_fds[0]);
+		close(to_cgi_fds[1]);
 		return -1;
 		break;
 	default: {
@@ -1176,6 +1187,7 @@ static int mod_cgi_patch_connection(server *srv, connection *con, plugin_data *p
 	plugin_config *s = p->config_storage[0];
 
 	PATCH(cgi);
+	PATCH(execute_x_only);
 
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
@@ -1191,6 +1203,8 @@ static int mod_cgi_patch_connection(server *srv, connection *con, plugin_data *p
 
 			if (buffer_is_equal_string(du->key, CONST_STR_LEN("cgi.assign"))) {
 				PATCH(cgi);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("cgi.execute-x-only"))) {
+				PATCH(execute_x_only);
 			}
 		}
 	}
@@ -1203,6 +1217,7 @@ URIHANDLER_FUNC(cgi_is_handled) {
 	size_t k, s_len;
 	plugin_data *p = p_d;
 	buffer *fn = con->physical.path;
+	stat_cache_entry *sce = NULL;
 
 	if (con->mode != DIRECT) return HANDLER_GO_ON;
 
@@ -1210,6 +1225,10 @@ URIHANDLER_FUNC(cgi_is_handled) {
 
 	mod_cgi_patch_connection(srv, con, p);
 
+	if (HANDLER_ERROR == stat_cache_get_entry(srv, con, con->physical.path, &sce)) return HANDLER_GO_ON;
+	if (!S_ISREG(sce->st.st_mode)) return HANDLER_GO_ON;
+	if (p->conf.execute_x_only == 1 && (sce->st.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) == 0) return HANDLER_GO_ON;
+
 	s_len = fn->used - 1;
 
 	for (k = 0; k < p->conf.cgi->used; k++) {
@@ -1369,6 +1388,7 @@ SUBREQUEST_FUNC(mod_cgi_handle_subrequest) {
 }
 
 
+int mod_cgi_plugin_init(plugin *p);
 int mod_cgi_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
 	p->name        = buffer_init_string("cgi");
