@@ -1,3 +1,12 @@
+#include "buffer.h"
+#include "server.h"
+#include "log.h"
+#include "plugin.h"
+#include "response.h"
+
+#include "mod_cml.h"
+#include "mod_cml_funcs.h"
+
 #include <sys/stat.h>
 #include <time.h>
 
@@ -8,15 +17,6 @@
 #include <dirent.h>
 #include <stdio.h>
 
-#include "buffer.h"
-#include "server.h"
-#include "log.h"
-#include "plugin.h"
-#include "response.h"
-
-#include "mod_cml.h"
-#include "mod_cml_funcs.h"
-
 #ifdef USE_OPENSSL
 # include <openssl/md5.h>
 #else
@@ -93,7 +93,7 @@ int f_file_mtime(lua_State *L) {
 	return 1;
 }
 
-int f_dir_files_iter(lua_State *L) {
+static int f_dir_files_iter(lua_State *L) {
 	DIR *d;
 	struct dirent *de;
 
@@ -211,7 +211,7 @@ int f_memcache_exists(lua_State *L) {
 	}
 
 	if (NULL == (r = mc_aget(mc,
-				 lua_tostring(L, 1), lua_strlen(L, 1)))) {
+				 (char*) lua_tostring(L, 1), lua_strlen(L, 1)))) {
 
 		lua_pushboolean(L, 0);
 		return 1;
@@ -248,7 +248,7 @@ int f_memcache_get_string(lua_State *L) {
 	}
 
 	if (NULL == (r = mc_aget(mc,
-				 lua_tostring(L, 1), lua_strlen(L, 1)))) {
+				 (char*) lua_tostring(L, 1), lua_strlen(L, 1)))) {
 		lua_pushnil(L);
 		return 1;
 	}
@@ -285,7 +285,7 @@ int f_memcache_get_long(lua_State *L) {
 	}
 
 	if (NULL == (r = mc_aget(mc,
-				 lua_tostring(L, 1), lua_strlen(L, 1)))) {
+				 (char*) lua_tostring(L, 1), lua_strlen(L, 1)))) {
 		lua_pushnil(L);
 		return 1;
 	}
