@@ -1,20 +1,17 @@
+#include "plugin.h"
+#include "log.h"
+
 #include <string.h>
 #include <stdlib.h>
 
 #include <stdio.h>
 
-#include "plugin.h"
-#include "log.h"
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #ifdef HAVE_VALGRIND_VALGRIND_H
-#include <valgrind/valgrind.h>
+# include <valgrind/valgrind.h>
 #endif
 
 #ifndef __WIN32
-#include <dlfcn.h>
+# include <dlfcn.h>
 #endif
 /*
  *
@@ -131,12 +128,12 @@ int plugins_load(server *srv) {
 
 		buffer_copy_string_buffer(srv->tmp_buf, srv->srvconf.modules_dir);
 
-		buffer_append_string(srv->tmp_buf, "/");
+		buffer_append_string_len(srv->tmp_buf, CONST_STR_LEN("/"));
 		buffer_append_string(srv->tmp_buf, modules);
 #if defined(__WIN32) || defined(__CYGWIN__)
-		buffer_append_string(srv->tmp_buf, ".dll");
+		buffer_append_string_len(srv->tmp_buf, CONST_STR_LEN(".dll"));
 #else
-		buffer_append_string(srv->tmp_buf, ".so");
+		buffer_append_string_len(srv->tmp_buf, CONST_STR_LEN(".so"));
 #endif
 
 		p = plugin_init();
@@ -173,7 +170,7 @@ int plugins_load(server *srv) {
 #endif
 		buffer_reset(srv->tmp_buf);
 		buffer_copy_string(srv->tmp_buf, modules);
-		buffer_append_string(srv->tmp_buf, "_plugin_init");
+		buffer_append_string_len(srv->tmp_buf, CONST_STR_LEN("_plugin_init"));
 
 #ifdef __WIN32
 		init = GetProcAddress(p->lib, srv->tmp_buf->ptr);
@@ -197,7 +194,7 @@ int plugins_load(server *srv) {
 
 #else
 #if 1
-		init = (int (*)(plugin *))dlsym(p->lib, srv->tmp_buf->ptr);
+		init = (int (*)(plugin *))(intptr_t)dlsym(p->lib, srv->tmp_buf->ptr);
 #else
 		*(void **)(&init) = dlsym(p->lib, srv->tmp_buf->ptr);
 #endif
