@@ -1,4 +1,7 @@
-#define _GNU_SOURCE
+#include "log.h"
+#include "stat_cache.h"
+#include "fdevent.h"
+#include "etag.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -11,13 +14,8 @@
 #include <fcntl.h>
 #include <assert.h>
 
-#include "log.h"
-#include "stat_cache.h"
-#include "fdevent.h"
-#include "etag.h"
-
 #ifdef HAVE_ATTR_ATTRIBUTES_H
-#include <attr/attributes.h>
+# include <attr/attributes.h>
 #endif
 
 #ifdef HAVE_FAM_H
@@ -36,7 +34,7 @@
 #endif
 
 #ifndef HAVE_LSTAT
-#define lstat stat
+# define lstat stat
 #endif
 
 #if 0
@@ -489,6 +487,12 @@ handler_t stat_cache_get_entry(server *srv, connection *con, buffer *name, stat_
 
 
 	if (S_ISREG(st.st_mode)) {
+		/* fix broken stat/open for symlinks to reg files with appended slash on freebsd,osx */
+		if (name->ptr[name->used-2] == '/') {
+			errno = ENOTDIR;
+			return HANDLER_ERROR;
+		}
+
 		/* try to open the file to check if we can read it */
 		if (-1 == (fd = open(name->ptr, O_RDONLY))) {
 			return HANDLER_ERROR;
@@ -497,11 +501,9 @@ handler_t stat_cache_get_entry(server *srv, connection *con, buffer *name, stat_
 	}
 
 	if (NULL == sce) {
-		int osize = 0;
-
-		if (sc->files) {
-			osize = sc->files->size;
-		}
+#ifdef DEBUG_STAT_CACHE
+		int osize = splaytree_size(sc->files);
+#endif
 
 		sce = stat_cache_entry_init();
 		buffer_copy_string_buffer(sce->name, name);
@@ -595,29 +597,31 @@ handler_t stat_cache_get_entry(server *srv, connection *con, buffer *name, stat_
 	if (S_ISREG(st.st_mode)) {
 		/* determine mimetype */
 		buffer_reset(sce->content_type);
-
-		for (k = 0; k < con->conf.mimetypes->used; k++) {
-			data_string *ds = (data_string *)con->conf.mimetypes->data[k];
-			buffer *type = ds->key;
-
-			if (type->used == 0) continue;
-
-			/* check if the right side is the same */
-			if (type->used > name->used) continue;
-
-			if (0 == strncasecmp(name->ptr + name->used - type->used, type->ptr, type->used - 1)) {
-				buffer_copy_string_buffer(sce->content_type, ds->value);
-				break;
-			}
-		}
- 		etag_create(sce->etag, &(sce->st), con->etag_flags);
 #ifdef HAVE_XATTR
-		if (con->conf.use_xattr && buffer_is_empty(sce->content_type)) {
+		if (con->conf.use_xattr) {
 			stat_cache_attr_get(sce->content_type, name->ptr);
 		}
 #endif
+		/* xattr did not set a content-type. ask the config */
+		if (buffer_is_empty(sce->content_type)) {
+			for (k = 0; k < con->conf.mimetypes->used; k++) {
+				data_string *ds = (data_string *)con->conf.mimetypes->data[k];
+				buffer *type = ds->key;
+
+				if (type->used == 0) continue;
+
+				/* check if the right side is the same */
+				if (type->used > name->used) continue;
+
+				if (0 == strncasecmp(name->ptr + name->used - type->used, type->ptr, type->used - 1)) {
+					buffer_copy_string_buffer(sce->content_type, ds->value);
+					break;
+				}
+			}
+		}
+		etag_create(sce->etag, &(sce->st), con->etag_flags);
 	} else if (S_ISDIR(st.st_mode)) {
- 		etag_create(sce->etag, &(sce->st), con->etag_flags);
+		etag_create(sce->etag, &(sce->st), con->etag_flags);
 	}
 
 #ifdef HAVE_FAM_H
